module SourceThih where

import Testbed
import HaskellPrims
import HaskellPrelude
import HaskellMaybe
import HaskellList

-----------------------------------------------------------------------------
-- Test Framework:

main     :: IO ()
main      = test imports (thihDefns ++ thihMems)

saveThih :: IO ()
saveThih  = save "HaskellThih" imports thihDefns

imports  :: [Assump]
imports   = defnsHaskellPrims ++
            defnsHaskellPrelude ++
            defnsHaskellMaybe ++
            defnsHaskellList

-----------------------------------------------------------------------------
-- Test Program:

thihDefns :: [BindGroup]
thihDefns 
 = map toBg
   [[("enumId",
      Just (Forall []
             ([] :=> 
              (tInt `fn` tId))),
      [([PVar "n"],
        ap [evar "++", elit (LitStr "v"), ap [econst showMfun, evar "n"]])])],
    [("tChar",
      Nothing,
      [([],
        ap [econst tConCfun, ap [econst tyconCfun, elit (LitStr "Char"), econst starCfun]])])],
    [("tArrow",
      Nothing,
      [([],
        ap [econst tConCfun, ap [econst tyconCfun, elit (LitStr "(->)"), ap [econst kfunCfun, econst starCfun, ap [econst kfunCfun, econst starCfun, econst starCfun]]]])])],
    [("fn",
      Just (Forall []
             ([] :=> 
              (tType `fn` tType `fn` tType))),
      [([PVar "a", PVar "b"],
        ap [econst tApCfun, ap [econst tApCfun, evar "tArrow", evar "a"], evar "b"])])],
    [("nullSubst",
      Just (Forall []
             ([] :=> 
              tSubst)),
      [([],
        econst nilCfun)])],
    [("+->",
      Just (Forall []
             ([] :=> 
              (tTyvar `fn` tType `fn` tSubst))),
      [([PVar "u", PVar "t"],
        eCons (ap [econst tup2Cfun, evar "u", evar "t"])
        eNil)])],
    [("@@",
      Just (Forall []
             ([] :=> 
              (tSubst `fn` tSubst `fn` tSubst))),
      [([PVar "s1", PVar "s2"],
        ap [evar "++", eCompFrom (PCon tup2Cfun [PVar "u", PVar "t"]) (evar "s2")
                       (eListRet (ap [econst tup2Cfun, evar "u", ap [econst applyMfun, evar "s1", evar "t"]])), evar "s1"])])],
    [("merge",
      Just (Forall []
             ([] :=> 
              (tSubst `fn` tSubst `fn` TAp tMaybe tSubst))),
      [([PVar "s1", PVar "s2"],
        elet [[("dom",
                Nothing,
                [([PVar "s"],
                  ap [evar "map", evar "fst", evar "s"])])],
              [("s",
                Nothing,
                [([],
                  ap [evar "++", evar "s1", evar "s2"])])],
              [("agree",
                Nothing,
                [([],
                  ap [evar "all", elambda ([PVar "v"],
                                           ap [econst eqMfun, ap [econst applyMfun, evar "s1", ap [econst tVarCfun, evar "v"]], ap [econst applyMfun, evar "s2", ap [econst tVarCfun, evar "v"]]]), ap [evar "intersect", ap [evar "dom", evar "s1"], ap [evar "dom", evar "s2"]]])])]]
             (eif (evar "agree")
                  (ap [econst justCfun, evar "s"])
                  (econst nothingCfun)))])],
    [("varBind",
      Just (Forall []
             ([] :=> 
              (tTyvar `fn` tType `fn` TAp tMaybe tSubst))),
      [([PVar "u", PVar "t"],
        eguarded [(ap [econst eqMfun, evar "t", ap [econst tVarCfun, evar "u"]],
                   ap [econst justCfun, evar "nullSubst"]),
                  (ap [evar "elem", evar "u", ap [econst tvMfun, evar "t"]],
                   econst nothingCfun),
                  (ap [econst eqMfun, ap [econst kindMfun, evar "u"], ap [econst kindMfun, evar "t"]],
                   ap [econst justCfun, ap [evar "+->", evar "u", evar "t"]]),
                  (evar "otherwise",
                   econst nothingCfun)])])],
    [("mgu",
      Just (Forall []
             ([] :=> 
              (tType `fn` tType `fn` TAp tMaybe tSubst))),
      [([PCon tApCfun [PVar "l", PVar "r"], PCon tApCfun [PVar "l'", PVar "r'"]],
        eCompFrom (PVar "s1") (ap [evar "mgu", evar "l", evar "l'"])
        (eCompFrom (PVar "s2") (ap [evar "mgu", ap [econst applyMfun, evar "s1", evar "r"], ap [econst applyMfun, evar "s1", evar "r'"]])
         (ap [econst justCfun, ap [evar "@@", evar "s2", evar "s1"]]))),
       ([PCon tVarCfun [PVar "u"], PVar "t"],
        ap [evar "varBind", evar "u", evar "t"]),
       ([PVar "t", PCon tVarCfun [PVar "u"]],
        ap [evar "varBind", evar "u", evar "t"]),
       ([PCon tConCfun [PVar "tc1"], PCon tConCfun [PVar "tc2"]],
        eguarded [(ap [econst eqMfun, evar "tc1", evar "tc2"],
                   ap [econst justCfun, evar "nullSubst"])]),
       ([PVar "t1", PVar "t2"],
        econst nothingCfun)])],
    [("match",
      Just (Forall []
             ([] :=> 
              (tType `fn` tType `fn` TAp tMaybe tSubst))),
      [([PCon tApCfun [PVar "l", PVar "r"], PCon tApCfun [PVar "l'", PVar "r'"]],
        eCompFrom (PVar "sl") (ap [evar "match", evar "l", evar "l'"])
        (eCompFrom (PVar "sr") (ap [evar "match", evar "r", evar "r'"])
         (ap [evar "merge", evar "sl", evar "sr"]))),
       ([PCon tVarCfun [PVar "u"], PVar "t"],
        eguarded [(ap [econst eqMfun, ap [econst kindMfun, evar "u"], ap [econst kindMfun, evar "t"]],
                   ap [econst returnMfun, ap [evar "+->", evar "u", evar "t"]])]),
       ([PCon tConCfun [PVar "tc1"], PCon tConCfun [PVar "tc2"]],
        eguarded [(ap [econst eqMfun, evar "tc1", evar "tc2"],
                   ap [econst returnMfun, evar "nullSubst"])]),
       ([PVar "t1", PVar "t2"],
        econst nothingCfun)])],
    [("bySuper",
      Just (Forall []
             ([] :=> 
              (tPred `fn` TAp tList tPred))),
      [([PAs "p" (PCon isInCfun [PVar "c", PVar "t"])],
        elet [[("supers",
                Nothing,
                [([],
                  eCompFrom (PVar "c'") (ap [econst superSfun, evar "c"])
                  (eListRet (ap [econst isInCfun, evar "c'", evar "t"])))])]]
             (ap [econst consCfun, evar "p", ap [evar "concat", ap [evar "map", evar "bySuper", evar "supers"]]]))])],
    [("matchPred",
      Just (Forall []
             ([] :=> 
              (tPred `fn` tPred `fn` TAp tMaybe tSubst))),
      [([PCon isInCfun [PVar "c", PVar "t"], PCon isInCfun [PVar "c'", PVar "t'"]],
        eguarded [(ap [econst eqMfun, evar "c", evar "c'"],
                   ap [evar "match", evar "t", evar "t'"]),
                  (evar "otherwise",
                   econst nothingCfun)])])],
    [("byInst",
      Just (Forall []
             ([] :=> 
              (tPred `fn` tInst `fn` TAp tMaybe (TAp tList tPred)))),
      [([PVar "p", PCon qualifyCfun [PVar "ps", PVar "h"]],
        eCompFrom (PVar "u") (ap [evar "matchPred", evar "h", evar "p"])
        (ap [econst returnMfun, ap [evar "map", ap [econst applyMfun, evar "u"], evar "ps"]]))])],
    [("reducePred",
      Just (Forall []
             ([] :=> 
              (tPred `fn` TAp tMaybe (TAp tList tPred)))),
      [([PAs "p" (PCon isInCfun [PVar "c", PVar "t"])],
        elet [[("poss",
                Nothing,
                [([],
                  ap [evar "map", ap [evar "byInst", evar "p"], ap [econst instsSfun, evar "c"]])])],
              [("|||",
                Nothing,
                [([PCon nothingCfun [], PVar "y"],
                  evar "y"),
                 ([PCon justCfun [PVar "x"], PVar "y"],
                  ap [econst justCfun, evar "x"])])]]
             (ap [evar "foldr", evar "|||", econst nothingCfun, evar "poss"]))])],
    [("||-",
      Just (Forall []
             ([] :=> 
              (TAp tList tPred `fn` tPred `fn` tBool))),
      [([PVar "ps", PVar "p"],
        ap [evar "||", ap [evar "any", ap [evar "elem", evar "p"], ap [evar "map", evar "bySuper", evar "ps"]], ecase (ap [evar "reducePred", evar "p"])
                                                                                                                      [(PCon nothingCfun [],
                                                                                                                        econst falseCfun),
                                                                                                                       (PCon justCfun [PVar "qs"],
                                                                                                                        ap [evar "all", ap [evar "||-", evar "ps"], evar "qs"])]])])],
    [("quantify",
      Just (Forall []
             ([] :=> 
              (TAp tList tTyvar `fn` TAp tQual tType `fn` tScheme))),
      [([PVar "vs", PVar "qt"],
        elet [[("vs'",
                Nothing,
                [([],
                  eCompFrom (PVar "v") (ap [econst tvMfun, evar "qt"])
                  (eCompGuard (ap [evar "elem", evar "v", evar "vs"])
                   (eListRet (evar "v"))))])],
              [("ks",
                Nothing,
                [([],
                  ap [evar "map", econst kindMfun, evar "vs'"])])],
              [("s",
                Nothing,
                [([],
                  ap [evar "zip", evar "vs'", ap [evar "map", econst tGenCfun, ap [econst enumFromMfun, elit (LitInt 0)]]])])]]
             (ap [econst forallCfun, evar "ks", ap [econst applyMfun, evar "s", evar "qt"]]))])],
    [("toScheme",
      Just (Forall []
             ([] :=> 
              (tType `fn` tScheme))),
      [([PVar "t"],
        ap [econst forallCfun, econst nilCfun, ap [econst qualifyCfun, econst nilCfun, evar "t"]])])],
    [("find",
      Just (Forall []
             ([] :=> 
              (tId `fn` TAp tList tAssump `fn` tScheme))),
      [([PVar "i", PVar "as"],
        ap [evar "head", eCompFrom (PCon assumeCfun [PVar "i'", PVar "sc"]) (evar "as")
                         (eCompGuard (ap [econst eqMfun, evar "i", evar "i'"])
                          (eListRet (evar "sc")))])])],
    [("runTI",
      Just (Forall [Star]
             ([] :=> 
              (TAp tTI (TGen 0) `fn` TGen 0))),
      [([PCon tICfun [PVar "c"]],
        elet [[("v1988",
                Nothing,
                [([], ap [evar "c", evar "nullSubst", elit (LitInt 0)])]),
               ("s",
                Nothing,
                [([], ecase (evar "v1988") [(PCon tup3Cfun [PVar "s", PVar "n", PVar "result"], evar "s")])]),
               ("n",
                Nothing,
                [([], ecase (evar "v1988") [(PCon tup3Cfun [PVar "s", PVar "n", PVar "result"], evar "n")])]),
               ("result",
                Nothing,
                [([], ecase (evar "v1988") [(PCon tup3Cfun [PVar "s", PVar "n", PVar "result"], evar "result")])])]]
             (evar "result"))])],
    [("getSubst",
      Just (Forall []
             ([] :=> 
              (TAp tTI tSubst))),
      [([],
        ap [econst tICfun, elambda ([PVar "s", PVar "n"],
                                    ap [econst tup3Cfun, evar "s", evar "n", evar "s"])])])],
    [("extSubst",
      Just (Forall []
             ([] :=> 
              (tSubst `fn` TAp tTI tUnit))),
      [([PVar "s'"],
        ap [econst tICfun, elambda ([PVar "s", PVar "n"],
                                    ap [econst tup3Cfun, ap [evar "@@", evar "s'", evar "s"], evar "n", econst unitCfun])])])],
    [("unify",
      Just (Forall []
             ([] :=> 
              (tType `fn` tType `fn` TAp tTI tUnit))),
      [([PVar "t1", PVar "t2"],
        eCompFrom (PVar "s") (evar "getSubst")
        (ecase (ap [evar "mgu", ap [econst applyMfun, evar "s", evar "t1"], ap [econst applyMfun, evar "s", evar "t2"]])
               [(PCon justCfun [PVar "u"],
                 ap [evar "extSubst", evar "u"]),
                (PCon nothingCfun [],
                 ap [evar "error", elit (LitStr "unification")])]))])],
    [("newTVar",
      Just (Forall []
             ([] :=> 
              (tKind `fn` TAp tTI tType))),
      [([PVar "k"],
        ap [econst tICfun, elambda ([PVar "s", PVar "n"],
                                    elet [[("v",
                                            Nothing,
                                            [([],
                                              ap [econst tyvarCfun, ap [evar "enumId", evar "n"], evar "k"])])]]
                                         (ap [econst tup3Cfun, evar "s", ap [econst plusMfun, evar "n", elit (LitInt 1)], ap [econst tVarCfun, evar "v"]]))])])],
    [("freshInst",
      Just (Forall []
             ([] :=> 
              (tScheme `fn` TAp tTI (TAp tQual tType)))),
      [([PCon forallCfun [PVar "ks", PVar "qt"]],
        eCompFrom (PVar "ts") (ap [evar "mapM", evar "newTVar", evar "ks"])
        (ap [econst returnMfun, ap [econst instMfun, evar "ts", evar "qt"]]))])],
    [("cNum",
      Nothing,
      [([],
        ap [econst classCfun, elit (LitStr "Num"), eNil, eNil])])],
    [("tiLit",
      Just (Forall []
             ([] :=> 
              (tLit `fn` TAp tTI (TAp (TAp tTuple2 (TAp tList tPred)) tType)))),
      [([PCon litCharCfun [PWildcard]],
        ap [econst returnMfun, ap [econst tup2Cfun, econst nilCfun, evar "tChar"]]),
       ([PCon litIntCfun [PWildcard]],
        eCompFrom (PVar "v") (ap [evar "newTVar", econst starCfun])
        (ap [econst returnMfun, ap [econst tup2Cfun, eCons (ap [econst isInCfun, evar "cNum", evar "v"])
                                                     eNil, evar "v"]]))])],
    [("tiPat",
      Just (Forall []
             ([] :=> 
              (tPat `fn` TAp tTI (TAp (TAp (TAp tTuple3 (TAp tList tPred)) (TAp tList tAssump)) tType)))),
      [([PCon pVarCfun [PVar "i"]],
        eCompFrom (PVar "v") (ap [evar "newTVar", econst starCfun])
        (ap [econst returnMfun, ap [econst tup3Cfun, econst nilCfun, eCons (ap [econst assumeCfun, evar "i", ap [evar "toScheme", evar "v"]])
                                                                     eNil, evar "v"]])),
       ([PCon pLitCfun [PVar "l"]],
        eCompFrom (PCon tup2Cfun [PVar "ps", PVar "t"]) (ap [evar "tiLit", evar "l"])
        (ap [econst returnMfun, ap [econst tup3Cfun, evar "ps", econst nilCfun, evar "t"]])),
       ([PCon pConCfun [PCon assumeCfun [PVar "i", PVar "sc"], PVar "pats"]],
        eCompFrom (PCon tup3Cfun [PVar "ps", PVar "as", PVar "ts"]) (ap [evar "tiPats", evar "pats"])
        (eCompFrom (PVar "t'") (ap [evar "newTVar", econst starCfun])
         (eCompFrom (PCon qualifyCfun [PVar "qs", PVar "t"]) (ap [evar "freshInst", evar "sc"])
          (eCompFrom PWildcard (ap [evar "unify", evar "t", ap [evar "foldr", evar "fn", evar "t'", evar "ts"]])
           (ap [econst returnMfun, ap [econst tup3Cfun, ap [evar "++", evar "ps", evar "qs"], evar "as", evar "t'"]])))))]),
     ("tiPats",
      Just (Forall []
             ([] :=> 
              (TAp tList tPat `fn` TAp tTI (TAp (TAp (TAp tTuple3 (TAp tList tPred)) (TAp tList tAssump)) (TAp tList tType))))),
      [([PVar "pats"],
        eCompFrom (PVar "psasts") (ap [evar "mapM", evar "tiPat", evar "pats"])
        (eCompLet [[("ps",
                     Nothing,
                     [([],
                       eCompFrom (PCon tup3Cfun [PVar "ps", PWildcard, PWildcard]) (evar "psasts")
                       (eCompFrom (PVar "p") (evar "ps")
                        (eListRet (evar "p"))))])],
                   [("as",
                     Nothing,
                     [([],
                       eCompFrom (PCon tup3Cfun [PWildcard, PVar "as", PWildcard]) (evar "psasts")
                       (eCompFrom (PVar "a") (evar "as")
                        (eListRet (evar "a"))))])],
                   [("ts",
                     Nothing,
                     [([],
                       eCompFrom (PCon tup3Cfun [PWildcard, PWildcard, PVar "t"]) (evar "psasts")
                       (eListRet (evar "t")))])]]
         (ap [econst returnMfun, ap [econst tup3Cfun, evar "ps", evar "as", evar "ts"]])))])],
    [("defaults",
      Nothing,
      [([],
        econst nilCfun)])],
    [("numClasses",
      Nothing,
      [([],
        econst nilCfun)])],
    [("stdClasses",
      Nothing,
      [([],
        econst nilCfun)])],
    [("defs",
      Just (Forall []
             ([] :=> 
              (tTyvar `fn` TAp tList tPred `fn` TAp tList tType))),
      [([PVar "v", PVar "qs"],
        elet [[("cs",
                Nothing,
                [([],
                  eCompFrom (PCon isInCfun [PVar "c", PVar "t"]) (evar "qs")
                  (eListRet (evar "c")))])],
              [("ts",
                Nothing,
                [([],
                  eCompFrom (PCon isInCfun [PVar "c", PVar "t"]) (evar "qs")
                  (eListRet (evar "t")))])]]
             (eCompGuard (ap [evar "all", ap [econst eqMfun, ap [econst tVarCfun, evar "v"]], evar "ts"])
              (eCompGuard (ap [evar "all", ap [evar "flip", evar "elem", evar "stdClasses"], evar "cs"])
               (eCompGuard (ap [evar "any", ap [evar "flip", evar "elem", evar "numClasses"], evar "cs"])
                (eCompFrom (PVar "t") (evar "defaults")
                 (eCompGuard (ap [evar "and", eCompFrom (PVar "c") (evar "cs")
                                              (eListRet (ap [evar "||-", econst nilCfun, ap [econst isInCfun, evar "c", evar "t"]]))])
                  (eListRet (evar "t"))))))))])],
    [("ambig",
      Just (Forall []
             ([] :=> 
              (TAp tList tTyvar `fn` TAp tList tPred `fn` TAp tList (TAp (TAp (TAp tTuple3 tTyvar) (TAp tList tPred)) (TAp tList tType))))),
      [([PVar "vs", PVar "ps"],
        eCompFrom (PVar "v") (ap [evar "\\\\", ap [econst tvMfun, evar "ps"], evar "vs"])
        (eCompLet [[("qs",
                     Nothing,
                     [([],
                       eCompFrom (PVar "p") (evar "ps")
                       (eCompGuard (ap [evar "elem", evar "v", ap [econst tvMfun, evar "p"]])
                        (eListRet (evar "p"))))])]]
         (eListRet (ap [econst tup3Cfun, evar "v", evar "qs", ap [evar "defs", evar "v", evar "qs"]]))))])],
    [("useDefaults",
      Just (Forall []
             ([] :=> 
              (TAp tList tTyvar `fn` TAp tList tPred `fn` TAp tList tPred))),
      [([PVar "vs", PVar "ps"],
        elet [[("ams",
                Nothing,
                [([],
                  ap [evar "ambig", evar "vs", evar "ps"])])],
              [("tss",
                Nothing,
                [([],
                  eCompFrom (PCon tup3Cfun [PVar "v", PVar "qs", PVar "ts"]) (evar "ams")
                  (eListRet (evar "ts")))])],
              [("ps'",
                Nothing,
                [([],
                  eCompFrom (PCon tup3Cfun [PVar "v", PVar "qs", PVar "ts"]) (evar "ams")
                  (eCompFrom (PVar "p") (evar "qs")
                   (eListRet (evar "p"))))])]]
             (eguarded [(ap [evar "any", evar "null", evar "tss"],
                         ap [evar "error", elit (LitStr "ambiguity")]),
                        (evar "otherwise",
                         ap [evar "\\\\", evar "ps", evar "ps'"])]))])],
    [("simplify",
      Just (Forall []
             ([] :=> 
              (TAp tList tPred `fn` TAp tList tPred `fn` TAp tList tPred))),
      [([PVar "rs", PCon nilCfun []],
        evar "rs"),
       ([PVar "rs", PCon consCfun [PVar "p", PVar "ps"]],
        elet [[("qs",
                Nothing,
                [([],
                  ap [evar "bySuper", evar "p"])])],
              [("\\\\",
                Nothing,
                [([PVar "rs", PVar "qs"],
                  eCompFrom (PVar "r") (evar "rs")
                  (eCompGuard (ap [evar "notElem", evar "r", evar "qs"])
                   (eListRet (evar "r"))))])]]
             (ap [evar "simplify", ap [econst consCfun, evar "p", ap [evar "\\\\", evar "rs", evar "qs"]], ap [evar "\\\\", evar "ps", evar "qs"]]))])],
    [("inHnf",
      Just (Forall []
             ([] :=> 
              (tPred `fn` tBool))),
      [([PCon isInCfun [PVar "c", PVar "t"]],
        elet [[("hnf",
                Nothing,
                [([PCon tVarCfun [PVar "v"]],
                  econst trueCfun),
                 ([PCon tConCfun [PVar "tc"]],
                  econst falseCfun),
                 ([PCon tApCfun [PVar "t", PWildcard]],
                  ap [evar "hnf", evar "t"])])]]
             (ap [evar "hnf", evar "t"]))])],
    [("toHnfs",
      Just (Forall []
             ([] :=> 
              (TAp tList tPred `fn` TAp tList tPred))),
      [([],
        ap [evar ".", evar "concat", ap [evar "map", evar "toHnf"]])]),
     ("toHnf",
      Just (Forall []
             ([] :=> 
              (tPred `fn` TAp tList tPred))),
      [([PVar "p"],
        eif (ap [evar "inHnf", evar "p"])
            (eCons (evar "p")
            eNil)
            (ecase (ap [evar "reducePred", evar "p"])
                   [(PCon nothingCfun [],
                     ap [evar "error", elit (LitStr "context reduction")]),
                    (PCon justCfun [PVar "ps"],
                     ap [evar "toHnfs", evar "ps"])]))])],
    [("split",
      Just (Forall []
             ([] :=> 
              (TAp tList tTyvar `fn` TAp tList tPred `fn` TAp (TAp tTuple2 (TAp tList tPred)) (TAp tList tPred)))),
      [([PVar "fs"],
        ap [evar ".", ap [evar "partition", ap [evar ".", ap [evar "all", ap [evar "flip", evar "elem", evar "fs"]], econst tvMfun]], ap [evar ".", ap [evar "simplify", econst nilCfun], evar "toHnfs"]])])],
    [("reduce",
      Just (Forall []
             ([] :=> 
              (TAp tList tTyvar `fn` TAp tList tTyvar `fn` TAp tList tPred `fn` TAp (TAp tTuple2 (TAp tList tPred)) (TAp tList tPred)))),
      [([PVar "fs", PVar "gs", PVar "ps"],
        elet [[("v1989",
                Nothing,
                [([], ap [evar "split", evar "fs", evar "ps"])]),
               ("ds",
                Nothing,
                [([], ecase (evar "v1989") [(PCon tup2Cfun [PVar "ds", PVar "rs"], evar "ds")])]),
               ("rs",
                Nothing,
                [([], ecase (evar "v1989") [(PCon tup2Cfun [PVar "ds", PVar "rs"], evar "rs")])])],
              [("rs'",
                Nothing,
                [([],
                  ap [evar "useDefaults", ap [evar "++", evar "fs", evar "gs"], evar "rs"])])]]
             (ap [econst tup2Cfun, evar "ds", evar "rs'"]))])],
    [("tiSeq",
      Just (Forall [Star]
             ([] :=> 
              (tInfer (TGen 0) (TAp tList tAssump) `fn` tInfer (TAp tList (TGen 0)) (TAp tList tAssump)))),
      [([PVar "ti", PVar "as", PCon nilCfun []],
        ap [econst returnMfun, ap [econst tup2Cfun, econst nilCfun, econst nilCfun]]),
       ([PVar "ti", PVar "as", PCon consCfun [PVar "bs", PVar "bss"]],
        eCompFrom (PCon tup2Cfun [PVar "ps", PVar "as'"]) (ap [evar "ti", evar "as", evar "bs"])
        (eCompFrom (PCon tup2Cfun [PVar "qs", PVar "as''"]) (ap [evar "tiSeq", evar "ti", ap [evar "++", evar "as'", evar "as"], evar "bss"])
         (ap [econst returnMfun, ap [econst tup2Cfun, ap [evar "++", evar "ps", evar "qs"], ap [evar "++", evar "as''", evar "as'"]]])))])],
    [("restricted",
      Just (Forall []
             ([] :=> 
              (TAp tList tImpl `fn` tBool))),
      [([PVar "bs"],
        elet [[("simple",
                Nothing,
                [([PCon tup2Cfun [PVar "i", PVar "alts"]],
                  ap [evar "any", ap [evar ".", evar "null", evar "fst"], evar "alts"])])]]
             (ap [evar "any", evar "simple", evar "bs"]))])],
    [("tiExpr",
      Just (Forall []
             ([] :=> 
              tInfer tExpr tType)),
      [([PVar "as", PCon varCfun [PVar "i"]],
        eCompLet [[("sc",
                    Nothing,
                    [([],
                      ap [evar "find", evar "i", evar "as"])])]]
        (eCompFrom (PCon qualifyCfun [PVar "ps", PVar "t"]) (ap [evar "freshInst", evar "sc"])
         (ap [econst returnMfun, ap [econst tup2Cfun, evar "ps", evar "t"]]))),
       ([PVar "as", PCon constCfun [PCon assumeCfun [PVar "i", PVar "sc"]]],
        eCompFrom (PCon qualifyCfun [PVar "ps", PVar "t"]) (ap [evar "freshInst", evar "sc"])
        (ap [econst returnMfun, ap [econst tup2Cfun, evar "ps", evar "t"]])),
       ([PVar "as", PCon litCfun [PVar "l"]],
        eCompFrom (PCon tup2Cfun [PVar "ps", PVar "t"]) (ap [evar "tiLit", evar "l"])
        (ap [econst returnMfun, ap [econst tup2Cfun, evar "ps", evar "t"]])),
       ([PVar "as", PCon apCfun [PVar "e", PVar "f"]],
        eCompFrom (PCon tup2Cfun [PVar "ps", PVar "te"]) (ap [evar "tiExpr", evar "as", evar "e"])
        (eCompFrom (PCon tup2Cfun [PVar "qs", PVar "tf"]) (ap [evar "tiExpr", evar "as", evar "f"])
         (eCompFrom (PVar "t") (ap [evar "newTVar", econst starCfun])
          (eCompFrom PWildcard (ap [evar "unify", ap [evar "fn", evar "tf", evar "t"], evar "te"])
           (ap [econst returnMfun, ap [econst tup2Cfun, ap [evar "++", evar "ps", evar "qs"], evar "t"]]))))),
       ([PVar "as", PCon letCfun [PVar "bg", PVar "e"]],
        eCompFrom (PCon tup2Cfun [PVar "ps", PVar "as'"]) (ap [evar "tiBindGroup", evar "as", evar "bg"])
        (eCompFrom (PCon tup2Cfun [PVar "qs", PVar "t"]) (ap [evar "tiExpr", ap [evar "++", evar "as'", evar "as"], evar "e"])
         (ap [econst returnMfun, ap [econst tup2Cfun, ap [evar "++", evar "ps", evar "qs"], evar "t"]])))]),
     ("tiBindGroup",
      Just (Forall []
             ([] :=> 
              tInfer tBindGroup (TAp tList tAssump))),
      [([PVar "as", PCon tup2Cfun [PVar "es", PVar "iss"]],
        eCompLet [[("as'",
                    Nothing,
                    [([],
                      eCompFrom (PCon tup3Cfun [PVar "v", PVar "sc", PVar "alts"]) (evar "es")
                      (eListRet (ap [econst assumeCfun, evar "v", evar "sc"])))])]]
        (eCompFrom (PCon tup2Cfun [PVar "ps", PVar "as''"]) (ap [evar "tiSeq", evar "tiImpls", ap [evar "++", evar "as'", evar "as"], evar "iss"])
         (eCompFrom (PVar "qs") (ap [evar "mapM", ap [evar "tiExpl", ap [evar "++", evar "as''", ap [evar "++", evar "as'", evar "as"]]], evar "es"])
          (ap [econst returnMfun, ap [econst tup2Cfun, ap [evar "++", evar "ps", ap [evar "concat", evar "qs"]], ap [evar "++", evar "as''", evar "as'"]]]))))]),
     ("tiExpl",
      Just (Forall []
             ([] :=> 
              (TAp tList tAssump `fn` tExpl `fn` TAp tTI (TAp tList tPred)))),
      [([PVar "as", PCon tup3Cfun [PVar "i", PVar "sc", PVar "alts"]],
        eCompFrom (PCon qualifyCfun [PVar "qs", PVar "t"]) (ap [evar "freshInst", evar "sc"])
        (eCompFrom (PVar "ps") (ap [evar "tiAlts", evar "as", evar "alts", evar "t"])
         (eCompFrom (PVar "s") (evar "getSubst")
          (eCompLet [[("qs'",
                       Nothing,
                       [([],
                         ap [econst applyMfun, evar "s", evar "qs"])])],
                     [("t'",
                       Nothing,
                       [([],
                         ap [econst applyMfun, evar "s", evar "t"])])],
                     [("ps'",
                       Nothing,
                       [([],
                         eCompFrom (PVar "p") (ap [econst applyMfun, evar "s", evar "ps"])
                         (eCompGuard (ap [evar "not", ap [evar "||-", evar "qs'", evar "p"]])
                          (eListRet (evar "p"))))])],
                     [("fs",
                       Nothing,
                       [([],
                         ap [econst tvMfun, ap [econst applyMfun, evar "s", evar "as"]])])],
                     [("gs",
                       Nothing,
                       [([],
                         ap [evar "\\\\", ap [econst tvMfun, evar "t'"], evar "fs"])])],
                     [("v1990",
                       Nothing,
                       [([], ap [evar "reduce", evar "fs", evar "gs", evar "ps'"])]),
                      ("ds",
                       Nothing,
                       [([], ecase (evar "v1990") [(PCon tup2Cfun [PVar "ds", PVar "rs"], evar "ds")])]),
                      ("rs",
                       Nothing,
                       [([], ecase (evar "v1990") [(PCon tup2Cfun [PVar "ds", PVar "rs"], evar "rs")])])],
                     [("sc'",
                       Nothing,
                       [([],
                         ap [evar "quantify", evar "gs", ap [econst qualifyCfun, evar "qs'", evar "t'"]])])]]
           (eif (ap [econst neqMfun, evar "sc", evar "sc'"])
                (ap [evar "error", elit (LitStr "signature too general")])
                (eif (ap [evar "not", ap [evar "null", evar "rs"]])
                     (ap [evar "error", elit (LitStr "context too weak")])
                     (ap [econst returnMfun, evar "ds"])))))))]),
     ("tiAlts",
      Just (Forall []
             ([] :=> 
              (TAp tList tAssump `fn` TAp tList tAlt `fn` tType `fn` TAp tTI (TAp tList tPred)))),
      [([PVar "as", PVar "alts", PVar "t"],
        eCompFrom (PVar "psts") (ap [evar "mapM", ap [evar "tiAlt", evar "as"], evar "alts"])
        (eCompFrom PWildcard (ap [evar "mapM", ap [evar "unify", evar "t"], ap [evar "map", evar "snd", evar "psts"]])
         (ap [econst returnMfun, ap [evar "concat", ap [evar "map", evar "fst", evar "psts"]]])))]),
     ("tiAlt",
      Just (Forall []
             ([] :=> 
              tInfer tAlt tType)),
      [([PVar "as", PCon tup2Cfun [PVar "pats", PVar "e"]],
        eCompFrom (PCon tup3Cfun [PVar "ps", PVar "as'", PVar "ts"]) (ap [evar "tiPats", evar "pats"])
        (eCompFrom (PCon tup2Cfun [PVar "qs", PVar "t"]) (ap [evar "tiExpr", ap [evar "++", evar "as'", evar "as"], evar "e"])
         (ap [econst returnMfun, ap [econst tup2Cfun, ap [evar "++", evar "ps", evar "qs"], ap [evar "foldr", evar "fn", evar "t", evar "ts"]]])))]),
     ("tiImpls",
      Just (Forall []
             ([] :=> 
              tInfer (TAp tList tImpl) (TAp tList tAssump))),
      [([PVar "as", PVar "bs"],
        eCompFrom (PVar "ts") (ap [evar "mapM", elambda ([PWildcard],
                                                         ap [evar "newTVar", econst starCfun]), evar "bs"])
        (eCompLet [[("is",
                     Nothing,
                     [([],
                       ap [evar "map", evar "fst", evar "bs"])])],
                   [("scs",
                     Nothing,
                     [([],
                       ap [evar "map", evar "toScheme", evar "ts"])])],
                   [("as'",
                     Nothing,
                     [([],
                       ap [evar "++", ap [evar "zipWith", econst assumeCfun, evar "is", evar "scs"], evar "as"])])],
                   [("altss",
                     Nothing,
                     [([],
                       ap [evar "map", evar "snd", evar "bs"])])]]
         (eCompFrom (PVar "pss") (ap [evar "sequence", ap [evar "zipWith", ap [evar "tiAlts", evar "as'"], evar "altss", evar "ts"]])
          (eCompFrom (PVar "s") (evar "getSubst")
           (eCompLet [[("ps'",
                        Nothing,
                        [([],
                          ap [econst applyMfun, evar "s", ap [evar "concat", evar "pss"]])])],
                      [("ts'",
                        Nothing,
                        [([],
                          ap [econst applyMfun, evar "s", evar "ts"])])],
                      [("fs",
                        Nothing,
                        [([],
                          ap [econst tvMfun, ap [econst applyMfun, evar "s", evar "as"]])])],
                      [("vss",
                        Nothing,
                        [([],
                          ap [evar "map", econst tvMfun, evar "ts'"])])],
                      [("gs",
                        Nothing,
                        [([],
                          ap [evar "\\\\", ap [evar "foldr1", evar "union", evar "vss"], evar "fs"])])],
                      [("v1991",
                        Nothing,
                        [([], ap [evar "reduce", evar "fs", ap [evar "foldr1", evar "intersect", evar "vss"], evar "ps'"])]),
                       ("ds",
                        Nothing,
                        [([], ecase (evar "v1991") [(PCon tup2Cfun [PVar "ds", PVar "rs"], evar "ds")])]),
                       ("rs",
                        Nothing,
                        [([], ecase (evar "v1991") [(PCon tup2Cfun [PVar "ds", PVar "rs"], evar "rs")])])]]
            (eif (ap [evar "restricted", evar "bs"])
                 (elet [[("gs'",
                          Nothing,
                          [([],
                            ap [evar "\\\\", evar "gs", ap [econst tvMfun, evar "rs"]])])],
                        [("scs'",
                          Nothing,
                          [([],
                            ap [evar "map", ap [evar ".", ap [evar "quantify", evar "gs'"], ap [econst qualifyCfun, econst nilCfun]], evar "ts'"])])]]
                       (ap [econst returnMfun, ap [econst tup2Cfun, ap [evar "++", evar "ds", evar "rs"], ap [evar "zipWith", econst assumeCfun, evar "is", evar "scs'"]]]))
                 (elet [[("scs'",
                          Nothing,
                          [([],
                            ap [evar "map", ap [evar ".", ap [evar "quantify", evar "gs"], ap [econst qualifyCfun, evar "rs"]], evar "ts'"])])]]
                       (ap [econst returnMfun, ap [econst tup2Cfun, evar "ds", ap [evar "zipWith", econst assumeCfun, evar "is", evar "scs'"]]]))))))))])],
    [("topDefaults",
      Just (Forall []
             ([] :=> 
              (TAp tList tPred `fn` TAp tMaybe tSubst))),
      [([PVar "ps"],
        elet [[("ams",
                Nothing,
                [([],
                  ap [evar "ambig", econst nilCfun, evar "ps"])])],
              [("tss",
                Nothing,
                [([],
                  eCompFrom (PCon tup3Cfun [PVar "v", PVar "qs", PVar "ts"]) (evar "ams")
                  (eListRet (evar "ts")))])],
              [("vs",
                Nothing,
                [([],
                  eCompFrom (PCon tup3Cfun [PVar "v", PVar "qs", PVar "ts"]) (evar "ams")
                  (eListRet (evar "v")))])]]
             (eguarded [(ap [evar "any", evar "null", evar "tss"],
                         econst nothingCfun),
                        (evar "otherwise",
                         ap [econst justCfun, ap [evar "zip", evar "vs", ap [evar "map", evar "head", evar "tss"]]])]))])],
    [("tiProgram",
      Just (Forall []
             ([] :=> 
              (TAp tList tAssump `fn` tProgram `fn` TAp tList tAssump))),
      [([PVar "as", PVar "bgs"],
        ap [evar "$", evar "runTI", eCompFrom (PCon tup2Cfun [PVar "ps", PVar "as'"]) (ap [evar "tiSeq", evar "tiBindGroup", evar "as", evar "bgs"])
                                    (eCompFrom (PVar "s") (evar "getSubst")
                                     (eCompLet [[("v1992",
                                                  Nothing,
                                                  [([], ap [evar "split", econst nilCfun, ap [econst applyMfun, evar "s", evar "ps"]])]),
                                                 ("rs",
                                                  Nothing,
                                                  [([], ecase (evar "v1992") [(PCon tup2Cfun [PCon nilCfun [], PVar "rs"], evar "rs")])])]]
                                      (ecase (ap [evar "topDefaults", evar "rs"])
                                             [(PCon justCfun [PVar "s'"],
                                               ap [econst returnMfun, ap [econst applyMfun, ap [evar "@@", evar "s'", evar "s"], evar "as'"]]),
                                              (PCon nothingCfun [],
                                               ap [evar "error", elit (LitStr "top-level ambiguity")])])))])])]]

thihMems :: [BindGroup]
thihMems
 = map (\x -> toBg [x])
   [("v2001",
     Just (Forall []
            ([isIn1 cHasKind tTyvar] :=> 
             (tTyvar `fn` tKind))),
     [([PCon tyvarCfun [PVar "v", PVar "k"]],
       evar "k")]),
    ("v2003",
     Just (Forall []
            ([isIn1 cHasKind tTycon] :=> 
             (tTycon `fn` tKind))),
     [([PCon tyconCfun [PVar "v", PVar "k"]],
       evar "k")]),
    ("v2005",
     Just (Forall []
            ([isIn1 cHasKind tType] :=> 
             (tType `fn` tKind))),
     [([PCon tConCfun [PVar "tc"]],
       ap [econst kindMfun, evar "tc"]),
      ([PCon tVarCfun [PVar "u"]],
       ap [econst kindMfun, evar "u"]),
      ([PCon tApCfun [PVar "t", PWildcard]],
       ecase (ap [econst kindMfun, evar "t"])
             [(PCon kfunCfun [PWildcard, PVar "k"],
               evar "k")])]),
    ("v2007",
     Just (Forall []
            ([isIn1 cTypes tType] :=> 
             (tSubst `fn` tType `fn` tType))),
     [([PVar "s", PCon tVarCfun [PVar "u"]],
       ecase (ap [evar "lookup", evar "u", evar "s"])
             [(PCon justCfun [PVar "t"],
               evar "t"),
              (PCon nothingCfun [],
               ap [econst tVarCfun, evar "u"])]),
      ([PVar "s", PCon tApCfun [PVar "l", PVar "r"]],
       ap [econst tApCfun, ap [econst applyMfun, evar "s", evar "l"], ap [econst applyMfun, evar "s", evar "r"]]),
      ([PVar "s", PVar "t"],
       evar "t")]),
    ("v2008",
     Just (Forall []
            ([isIn1 cTypes tType] :=> 
             (tType `fn` TAp tList tTyvar))),
     [([PCon tVarCfun [PVar "u"]],
       eCons (evar "u")
       eNil),
      ([PCon tApCfun [PVar "l", PVar "r"]],
       ap [evar "union", ap [econst tvMfun, evar "l"], ap [econst tvMfun, evar "r"]]),
      ([PVar "t"],
       econst nilCfun)]),
    ("v2010",
     Just (Forall [Star]
            ([isIn1 cTypes (TGen 0)] :=> 
             (tSubst `fn` TAp tList (TGen 0) `fn` TAp tList (TGen 0)))),
     [([PVar "s"],
       ap [evar "map", ap [econst applyMfun, evar "s"]])]),
    ("v2011",
     Just (Forall [Star]
            ([isIn1 cTypes (TGen 0)] :=> 
             (TAp tList (TGen 0) `fn` TAp tList tTyvar))),
     [([],
       ap [evar ".", evar "nub", ap [evar ".", evar "concat", ap [evar "map", econst tvMfun]]])]),
    ("v2013",
     Just (Forall [Star]
            ([isIn1 cTypes (TGen 0)] :=> 
             (tSubst `fn` TAp tQual (TGen 0) `fn` TAp tQual (TGen 0)))),
     [([PVar "s", PCon qualifyCfun [PVar "ps", PVar "t"]],
       ap [econst qualifyCfun, ap [econst applyMfun, evar "s", evar "ps"], ap [econst applyMfun, evar "s", evar "t"]])]),
    ("v2014",
     Just (Forall [Star]
            ([isIn1 cTypes (TGen 0)] :=> 
             (TAp tQual (TGen 0) `fn` TAp tList tTyvar))),
     [([PCon qualifyCfun [PVar "ps", PVar "t"]],
       ap [evar "union", ap [econst tvMfun, evar "ps"], ap [econst tvMfun, evar "t"]])]),
    ("v2016",
     Just (Forall []
            ([isIn1 cTypes tPred] :=> 
             (tSubst `fn` tPred `fn` tPred))),
     [([PVar "s", PCon isInCfun [PVar "c", PVar "t"]],
       ap [econst isInCfun, evar "c", ap [econst applyMfun, evar "s", evar "t"]])]),
    ("v2017",
     Just (Forall []
            ([isIn1 cTypes tPred] :=> 
             (tPred `fn` TAp tList tTyvar))),
     [([PCon isInCfun [PVar "c", PVar "t"]],
       ap [econst tvMfun, evar "t"])]),
    ("v2019",
     Just (Forall []
            ([isIn1 cEq tClass] :=> 
             (tClass `fn` tClass `fn` tBool))),
     [([PVar "c", PVar "c'"],
       ap [econst eqMfun, ap [econst nameSfun, evar "c"], ap [econst nameSfun, evar "c'"]])]),
    ("v2022",
     Just (Forall []
            ([isIn1 cTypes tScheme] :=> 
             (tSubst `fn` tScheme `fn` tScheme))),
     [([PVar "s", PCon forallCfun [PVar "ks", PVar "qt"]],
       ap [econst forallCfun, evar "ks", ap [econst applyMfun, evar "s", evar "qt"]])]),
    ("v2023",
     Just (Forall []
            ([isIn1 cTypes tScheme] :=> 
             (tScheme `fn` TAp tList tTyvar))),
     [([PCon forallCfun [PVar "ks", PVar "qt"]],
       ap [econst tvMfun, evar "qt"])]),
    ("v2025",
     Just (Forall []
            ([isIn1 cTypes tAssump] :=> 
             (tSubst `fn` tAssump `fn` tAssump))),
     [([PVar "s", PCon assumeCfun [PVar "i", PVar "sc"]],
       ap [econst assumeCfun, evar "i", ap [econst applyMfun, evar "s", evar "sc"]])]),
    ("v2026",
     Just (Forall []
            ([isIn1 cTypes tAssump] :=> 
             (tAssump `fn` TAp tList tTyvar))),
     [([PCon assumeCfun [PVar "i", PVar "sc"]],
       ap [econst tvMfun, evar "sc"])]),
    ("v2028",
     Just (Forall [Star]
            ([isIn1 cMonad tTI] :=> 
             (TGen 0 `fn` TAp tTI (TGen 0)))),
     [([PVar "x"],
       ap [econst tICfun, elambda ([PVar "s", PVar "n"],
                                   ap [econst tup3Cfun, evar "s", evar "n", evar "x"])])]),
    ("v2029",
     Just (Forall [Star, Star]
            ([isIn1 cMonad tTI] :=> 
             (TAp tTI (TGen 0) `fn` (TGen 0 `fn` TAp tTI (TGen 1)) `fn` TAp tTI (TGen 1)))),
     [([PCon tICfun [PVar "c"], PVar "f"],
       ap [econst tICfun, elambda ([PVar "s", PVar "n"],
                                   elet [[("v2030",
                                           Nothing,
                                           [([], ap [evar "c", evar "s", evar "n"])]),
                                          ("s'",
                                           Nothing,
                                           [([], ecase (evar "v2030") [(PCon tup3Cfun [PVar "s'", PVar "m", PVar "x"], evar "s'")])]),
                                          ("m",
                                           Nothing,
                                           [([], ecase (evar "v2030") [(PCon tup3Cfun [PVar "s'", PVar "m", PVar "x"], evar "m")])]),
                                          ("x",
                                           Nothing,
                                           [([], ecase (evar "v2030") [(PCon tup3Cfun [PVar "s'", PVar "m", PVar "x"], evar "x")])])],
                                         [("v2031",
                                           Nothing,
                                           [([], ap [evar "f", evar "x"])]),
                                          ("fx",
                                           Nothing,
                                           [([], ecase (evar "v2031") [(PCon tICfun [PVar "fx"], evar "fx")])])]]
                                        (ap [evar "fx", evar "s'", evar "m"]))])]),
    ("v2035",
     Just (Forall []
            ([isIn1 cInstantiate tType] :=> 
             (TAp tList tType `fn` tType `fn` tType))),
     [([PVar "ts", PCon tApCfun [PVar "l", PVar "r"]],
       ap [econst tApCfun, ap [econst instMfun, evar "ts", evar "l"], ap [econst instMfun, evar "ts", evar "r"]]),
      ([PVar "ts", PCon tGenCfun [PVar "n"]],
       ap [evar "!!", evar "ts", evar "n"]),
      ([PVar "ts", PVar "t"],
       evar "t")]),
    ("v2037",
     Just (Forall [Star]
            ([isIn1 cInstantiate (TGen 0)] :=> 
             (TAp tList tType `fn` TAp tList (TGen 0) `fn` TAp tList (TGen 0)))),
     [([PVar "ts"],
       ap [evar "map", ap [econst instMfun, evar "ts"]])]),
    ("v2039",
     Just (Forall [Star]
            ([isIn1 cInstantiate (TGen 0)] :=> 
             (TAp tList tType `fn` TAp tQual (TGen 0) `fn` TAp tQual (TGen 0)))),
     [([PVar "ts", PCon qualifyCfun [PVar "ps", PVar "t"]],
       ap [econst qualifyCfun, ap [econst instMfun, evar "ts", evar "ps"], ap [econst instMfun, evar "ts", evar "t"]])]),
    ("v2041",
     Just (Forall []
            ([isIn1 cInstantiate tPred] :=> 
             (TAp tList tType `fn` tPred `fn` tPred))),
     [([PVar "ts", PCon isInCfun [PVar "c", PVar "t"]],
       ap [econst isInCfun, evar "c", ap [econst instMfun, evar "ts", evar "t"]])]),
    ("v2043",
     Just (Forall []
            ([isIn1 cEq tKind] :=> 
             (tKind `fn` tKind `fn` tBool))),
     [([PCon starCfun [], PCon starCfun []],
       econst trueCfun),
      ([PCon kfunCfun [PVar "v1978", PVar "v1977"], PCon kfunCfun [PVar "v1980", PVar "v1979"]],
       ap [evar "&&", ap [econst eqMfun, evar "v1978", evar "v1980"], ap [econst eqMfun, evar "v1977", evar "v1979"]]),
      ([PWildcard, PWildcard],
       econst falseCfun)]),
    ("v2046",
     Just (Forall []
            ([isIn1 cEq tType] :=> 
             (tType `fn` tType `fn` tBool))),
     [([PCon tVarCfun [PVar "v1979"], PCon tVarCfun [PVar "v1980"]],
       ap [econst eqMfun, evar "v1979", evar "v1980"]),
      ([PCon tConCfun [PVar "v1979"], PCon tConCfun [PVar "v1980"]],
       ap [econst eqMfun, evar "v1979", evar "v1980"]),
      ([PCon tApCfun [PVar "v1978", PVar "v1977"], PCon tApCfun [PVar "v1980", PVar "v1979"]],
       ap [evar "&&", ap [econst eqMfun, evar "v1978", evar "v1980"], ap [econst eqMfun, evar "v1977", evar "v1979"]]),
      ([PCon tGenCfun [PVar "v1979"], PCon tGenCfun [PVar "v1980"]],
       ap [econst eqMfun, evar "v1979", evar "v1980"]),
      ([PWildcard, PWildcard],
       econst falseCfun)]),
    ("v2049",
     Just (Forall []
            ([isIn1 cEq tTyvar] :=> 
             (tTyvar `fn` tTyvar `fn` tBool))),
     [([PCon tyvarCfun [PVar "v1978", PVar "v1977"], PCon tyvarCfun [PVar "v1980", PVar "v1979"]],
       ap [evar "&&", ap [econst eqMfun, evar "v1978", evar "v1980"], ap [econst eqMfun, evar "v1977", evar "v1979"]])]),
    ("v2052",
     Just (Forall []
            ([isIn1 cEq tTycon] :=> 
             (tTycon `fn` tTycon `fn` tBool))),
     [([PCon tyconCfun [PVar "v1978", PVar "v1977"], PCon tyconCfun [PVar "v1980", PVar "v1979"]],
       ap [evar "&&", ap [econst eqMfun, evar "v1978", evar "v1980"], ap [econst eqMfun, evar "v1977", evar "v1979"]])]),
    ("v2055",
     Just (Forall [Star]
            ([isIn1 cEq (TGen 0)] :=> 
             (TAp tQual (TGen 0) `fn` TAp tQual (TGen 0) `fn` tBool))),
     [([PCon qualifyCfun [PVar "v1978", PVar "v1977"], PCon qualifyCfun [PVar "v1980", PVar "v1979"]],
       ap [evar "&&", ap [econst eqMfun, evar "v1978", evar "v1980"], ap [econst eqMfun, evar "v1977", evar "v1979"]])]),
    ("v2058",
     Just (Forall []
            ([isIn1 cEq tPred] :=> 
             (tPred `fn` tPred `fn` tBool))),
     [([PCon isInCfun [PVar "v1978", PVar "v1977"], PCon isInCfun [PVar "v1980", PVar "v1979"]],
       ap [evar "&&", ap [econst eqMfun, evar "v1978", evar "v1980"], ap [econst eqMfun, evar "v1977", evar "v1979"]])]),
    ("v2061",
     Just (Forall []
            ([isIn1 cEq tScheme] :=> 
             (tScheme `fn` tScheme `fn` tBool))),
     [([PCon forallCfun [PVar "v1978", PVar "v1977"], PCon forallCfun [PVar "v1980", PVar "v1979"]],
       ap [evar "&&", ap [econst eqMfun, evar "v1978", evar "v1980"], ap [econst eqMfun, evar "v1977", evar "v1979"]])])]

-----------------------------------------------------------------------------
